package example;

import Log;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class MyCode implements AWTEventListener {

    private static class PopupAdapter extends MouseAdapter {
        private boolean dynamic;
        private JPopupMenu cachedMenu = null;
        private int invokes = 0;
        public PopupAdapter(boolean dynamic) {
            this.dynamic = dynamic;
        }
        public void mousePressed(MouseEvent ev) {
            if (ev.isPopupTrigger()) {
                JPopupMenu menu = getPopupMenu();
                menu.pack();
                menu.show((Component)ev.getSource(), ev.getX(), ev.getY());
            }
        }
        private JPopupMenu getPopupMenu() {
            JPopupMenu menu = cachedMenu;
            if (menu == null) {
                menu = new JPopupMenu();
                if (dynamic) {
                    menu.add(new JMenuItem("Invoked " + ++invokes + " times"));
                    menu.add(new JSeparator());
                }
                menu.add(new JMenuItem("Black"));
                menu.add(new JMenuItem("Blue"));
                menu.add(new JMenuItem("Orange"));
                JMenu submenu = new JMenu("Other");
                submenu.add(new JMenuItem("White"));
                submenu.add(new JMenuItem("Green"));
                menu.add(submenu);
                if (!dynamic)
                    cachedMenu = menu;
            }
            return menu;
        }
    }

    public void eventDispatched(AWTEvent event) {
        /*
        Object src = event.getSource();
        Class cls = src.getClass();
        String name = cls.getName();
        if (src instanceof Component)
            name += ":" + ((Component)src).getName();
        */
        Log.log(event.toString());
    }

    public static void main(String[] args) {
        args = Log.init(args);
        //System.setProperty("com.apple.macos.useScreenMenuBar","true");
        long mask = -1; /*(AWTEvent.ACTION_EVENT_MASK
                     | AWTEvent.ADJUSTMENT_EVENT_MASK 
                     | AWTEvent.COMPONENT_EVENT_MASK
                     | AWTEvent.CONTAINER_EVENT_MASK
                     | AWTEvent.FOCUS_EVENT_MASK
                     | AWTEvent.HIERARCHY_BOUNDS_EVENT_MASK
                     | AWTEvent.HIERARCHY_EVENT_MASK
                     | AWTEvent.INPUT_METHOD_EVENT_MASK
                     | AWTEvent.INVOCATION_EVENT_MASK
                     | AWTEvent.ITEM_EVENT_MASK
                     | AWTEvent.KEY_EVENT_MASK
                     | AWTEvent.MOUSE_EVENT_MASK
                     | AWTEvent.MOUSE_MOTION_EVENT_MASK
                     | AWTEvent.PAINT_EVENT_MASK
                     | AWTEvent.TEXT_EVENT_MASK
                     | AWTEvent.WINDOW_EVENT_MASK);*/
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        toolkit.addAWTEventListener(new MyCode(), mask);

        final JFrame frame = new JFrame("My Code");
        JPanel pane = new JPanel();
        pane.setName("My Pane");
        JLabel label = new JLabel("Static");
        label.setName("My Label");
        label.addMouseListener(new PopupAdapter(false));
        pane.add(label);
        label = new JLabel("Dynamic");
        label.setName("Dynamic");
        label.addMouseListener(new PopupAdapter(true));
        pane.add(label);
        JButton button = new JButton("Button");
        button.setName("My Button");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                JOptionPane.showMessageDialog(frame, "My Dialog Message");
            }
        });
        pane.add(button);
        JTextField tf = new JTextField("Text field");
        tf.setName("My Text Field");
        pane.add(tf);
        JComboBox cb = new JComboBox();
        for (int i=0;i < 20;i++)
            cb.addItem("Combo " + i);
        pane.add(cb);

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        ActionListener al = new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                System.out.println("Action: " + ev.getActionCommand());
            }
        };

        frame.setContentPane(pane);
        frame.setName("My Frame");
        JMenuBar menubar = new JMenuBar();
        menubar.setName("My Menu Bar");
        JMenu menu = new JMenu("File");
        menu.setName("My File Menu");

        JMenuItem mitem = new JMenuItem("Item 1");
        mitem.setName("My Menu Item (Item 1)"); menu.add(mitem);
        JMenu submenu = new JMenu("File submenu"); 
        submenu.setName("File submenu"); menu.add(submenu);
        mitem = new JMenuItem("Quit");
        mitem.setName("My Menu Item (Quit)"); menu.add(mitem);

        mitem = new JMenuItem("Submenu item");
        mitem.setName("Submenu item");
        mitem.addActionListener(al);
        submenu.add(mitem);

        JMenu menu2 = new JMenu("Edit");
        menu2.setName("Edit");
        mitem = new JMenuItem("Copy");
        mitem.setName("Copy"); menu2.add(mitem);

        menubar.add(menu);
        menubar.add(menu2);
        frame.setJMenuBar(menubar);
        frame.pack();
        frame.show();
    }
}
